// -----------------------------------
// Filename      : ButtonDebouncer.cpp
// Author        : Sven Maerivoet
// Last modified : 13/12/2017
// Target        : C++ (Arduino UNO)
//
// All rights reserved.
// -----------------------------------

/*
 Copyright 2016-2017 Sven Maerivoet
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
 
   http://www.apache.org/licenses/LICENSE-2.0
 
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/

#include "Arduino.h"
#include "ButtonDebouncer.h"

// ***********
// CONSTRUCTOR
// ***********

ButtonDebouncer::ButtonDebouncer()
{
	fNrOfButtons = 0;
	for (int i = 0; i < kMaxNrOfButtons; ++i) {
    fButtonPins[i] = 0;
		fButtonActives[i] = false;
    fLastDebounceTimes[i] = 0;
	}
  fDebounceDelay = kDefaultDebounceDelay;
  fLongPressDelay = kDefaultLongPressDelay;
  fVeryLongPressDelay = kDefaultVeryLongPressDelay;
  fExtremelyLongPressDelay = kDefaultExtremelyLongPressDelay;
}

// **************
// PUBLIC METHODS
// **************

void ButtonDebouncer::setDebounceDelay(int debounceDelay)
{
	fDebounceDelay = debounceDelay;
}

void ButtonDebouncer::setLongPressDelay(int longPressDelay)
{
	fLongPressDelay = longPressDelay;
}

void ButtonDebouncer::setVeryLongPressDelay(int veryLongPressDelay)
{
	fVeryLongPressDelay = veryLongPressDelay;
}

void ButtonDebouncer::setExtremelyLongPressDelay(int extremelyLongPressDelay)
{
	fExtremelyLongPressDelay = extremelyLongPressDelay;
}

void ButtonDebouncer::addButton(int pin)
{
  if (fNrOfButtons < kMaxNrOfButtons) {
  	pinMode(pin,INPUT);
    fButtonPins[fNrOfButtons] = pin;
    ++fNrOfButtons;
  }
}

ButtonDebouncer::EButtonState ButtonDebouncer::checkButton(int pin)
{
  // find the index corresponding to the requested button pin
  int buttonIndex = 0;
  bool buttonIndexFound = false;
  while ((buttonIndex < fNrOfButtons) && (!buttonIndexFound)) {
    if (fButtonPins[buttonIndex] == pin) {
      buttonIndexFound = true;
    }
    else {
      ++buttonIndex;
    }
  }

  if (!buttonIndexFound) {
    return kIdle;
  }

	EButtonState buttonState = kIdle;
  int reading = digitalRead(pin);
	unsigned long currentTime = millis();

	if (reading) {
		if (!fButtonActives[buttonIndex]) {
			fButtonActives[buttonIndex] = true;
			fLastDebounceTimes[buttonIndex] = currentTime;
		}
	}
	else {
		if (fButtonActives[buttonIndex]) {
			unsigned long pressedTime = currentTime - fLastDebounceTimes[buttonIndex];

			if (pressedTime > fVeryLongPressDelay) {
				buttonState = kVeryLongPressed;
			}
			else if (pressedTime > fLongPressDelay) {
				buttonState = kLongPressed;
			}
			else if (pressedTime > fDebounceDelay) {
				buttonState = kPressed;
			}

			fButtonActives[buttonIndex] = false;
		}
	}

	return buttonState;
}
